/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.compass;


import com.meterware.httpunit.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.Hashtable;


/**
 *  Given a zip code, determine associated city/state, or vice versa.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: ZipCodeSource.java,v 1.2 2003/09/08 05:22:45 glchen Exp $
 */
public class ZipCodeSource 
{
    public static Hashtable _state = null;
    
    protected static Hashtable _region = new Hashtable();
    protected static Hashtable _zipcode = new Hashtable();
    
    //=======================================================================
    // Consructors.
    //=======================================================================
    
    public ZipCodeSource()
    {
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Return the zipcode for specified region (city and state). */
    public synchronized static String getZipCode(String city, String state)
        throws Exception
    {
        String region = encodeRegion(city, state);
        // check mapping table first
        String ret = (String) _region.get(region);
        if (ret != null)
        {
            return ret;
        }

        // use address server by buffalo university
        String ipLocator = "http://www.cedar.buffalo.edu/AdServ/zip-search.html";
        WebConversation wc = new WebConversation();
        WebResponse wp = wc.getResponse(ipLocator);
        WebForm form = wp.getFormWithName("ctystzip");
        form.setParameter("ctystzip", city + " " + state);
        wp = form.submit();

        // manually parse the reply from usps
        BufferedReader in = new BufferedReader(
            new InputStreamReader(wp.getInputStream()));
        String line = in.readLine();
        while (line != null)
        {
            if (line.indexOf("city name") != -1)
            {
                int p1 = line.indexOf("<BR><BR>");
                int p2 = line.indexOf(" ", p1);
                ret = line.substring(p1+8, p2).trim();
            }
            line = in.readLine();
        }
        in.close();
        if (ret == null)
        {
            throw new Exception("City/State not found.");
        }

        // record the mapping and return the result
        _region.put(region, ret);
        _zipcode.put(ret, region);
        return ret;
    }

    /** Return the region (city and state) for specified zipcode. */
    public synchronized static String getRegion(String zipcode)
        throws Exception
    {
        String city = null;
        String state = null;
        String region = null;
        
        // check mapping table first
        region = (String) _zipcode.get(zipcode);
        if (region != null)
        {
            return region;
        }

        // use address server by buffalo university
        String ipLocator = "http://www.cedar.buffalo.edu/AdServ/zip-search.html";
        WebConversation wc = new WebConversation();
        WebResponse wp = wc.getResponse(ipLocator);
        WebForm form = wp.getFormWithName("ctystzip");
        form.setParameter("ctystzip", zipcode);
        wp = form.submit();

        // manually parse the reply from usps
        BufferedReader in = new BufferedReader(
            new InputStreamReader(wp.getInputStream()));
        String line = in.readLine();
        while (line != null)
        {
            if (line.indexOf("city name") != -1)
            {
                int p1 = line.indexOf("----<BR>");
                int p2 = line.indexOf("<BR>", p1+8);
                String[] fd = line.substring(p1+8, p2-1).split("[ ]+");
                int len = fd.length - 3;
                city = "";
                for (int i=0; i<len - 1; i++)
                {
                    city += fd[i] + " ";
                }
                city = city.trim();
                state = fd[fd.length-4];
                
            }
            line = in.readLine();
        }
        in.close();
        if (city == null)
        {
            throw new Exception("Zip code not found.");
        }

        // record the mapping and return the result
        region = encodeRegion(city, state);
        _zipcode.put(zipcode, region);
        _region.put(region, zipcode);
        return region;
    }

    /** Return the city for specified zipcode. */
    public synchronized static String getCity(String zipcode)
        throws Exception
    {
        return decodeCity(getRegion(zipcode));
    }

    /** Return the state for specified zipcode. */
    public synchronized static String getState(String zipcode)
        throws Exception
    {
        return decodeState(getRegion(zipcode));
    }

    /** Encode city and state into region representation. */
    private static String encodeRegion(String city, String state)
    {
        return city + "|" + state;
    }

    /** Decode city from region represtation. */
    private static String decodeCity(String region)
    {
        int p = region.indexOf("|");
        return region.substring(0, p);
    }

    /** Decode state from region represtation. */
    private static String decodeState(String region)
    {
        int p = region.indexOf("|");
        return region.substring(p+1);
    }

    /** Get full name of a state for specified abbreviation. */
    public static String getStateName(String abbr)
    {
        if (abbr == null)
        {
            throw new IllegalArgumentException("Null argument specified.");
        }
        if (_state == null)
        {
            initStateMapping();
        }
        String ret = null;
        Iterator itr = _state.keySet().iterator();
        while (itr.hasNext())
        {
            ret = (String) itr.next();
            if (abbr.equals(_state.get(ret)))
            {
                break;
            }
        }
        return ret;
    }

    /** Get abbr name of a state for specified full name. */
    public static String getStateAbbr(String name)
    {
        if (name == null)
        {
            throw new IllegalArgumentException("Null argument specified.");
        }
        if (_state == null)
        {
            initStateMapping();
        }
        return (String) _state.get(name);
    }

    /** Init state name to abbrev mapping. */
    private synchronized static void initStateMapping()
    {
        _state = new Hashtable();
        _state.put("Alabama", "AL");
        _state.put("Alaska", "AK");
        _state.put("Arizona", "AZ");
        _state.put("Arkansas", "AR");
        _state.put("California", "CA");
        _state.put("Colorado", "CO");
        _state.put("Connecticut", "CT");
        _state.put("Delaware", "DE");
        _state.put("Florida", "FL");
        _state.put("Georgia", "GA");
        _state.put("Hawaii", "HI");
        _state.put("Idaho", "ID");
        _state.put("Illinois", "IL");
        _state.put("Indiana", "IN");
        _state.put("Iowa", "IA");
        _state.put("Kansas", "KS");
        _state.put("Kentucky", "KY");
        _state.put("Louisiana", "LA");
        _state.put("Maine", "ME");
        _state.put("Maryland", "MD");
        _state.put("Massachusetts", "MA");
        _state.put("Michigan", "MI");
        _state.put("Minnesota", "MN");
        _state.put("Mississippi", "MS");
        _state.put("Missouri", "MO");
        _state.put("Montana", "MT");
        _state.put("Nebraska", "NE");
        _state.put("Nevada", "NV");
        _state.put("New Hampshire", "NH");
        _state.put("New Jersey", "NJ");
        _state.put("New Mexico", "NM");
        _state.put("New York", "NY");
        _state.put("North Carolina", "NC");
        _state.put("North Dakota", "ND");
        _state.put("Ohio", "OH");
        _state.put("Oklahoma", "OK");
        _state.put("Oregon", "OR");
        _state.put("Pennsylvania", "PA");
        _state.put("Rhode Island", "RI");
        _state.put("South Carolina", "SC");
        _state.put("South Dakota", "SD");
        _state.put("Tennessee", "TN");
        _state.put("Texas", "TX");
        _state.put("Utah", "UT");
        _state.put("Vermont", "VT");
        _state.put("Virginia", "VA");
        _state.put("Washington", "WA");
        _state.put("West Virginia", "WV");
        _state.put("Wisconsin", "WI");
        _state.put("Wyoming", "WY");
    }


    // testing
    public static void main(String[] args)
        throws Exception
    {
        System.out.println(ZipCodeSource.getZipCode("Hanover", "New Hampshire"));
        System.out.println(ZipCodeSource.getCity("03766"));
        System.out.println(ZipCodeSource.getState("03766"));
    }
}
