/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.snmp;


import serp.util.Options;

import solar.Client;
import solar.api.Request;
import solar.service.dan.Source;



/**
 *  A Solar source that polls access points to get MAC-IP mapping using SNMP.
 *  It current calls an external Perl script and parses its output, will be 
 *  nice to poll the AP using a Java library.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: SnmpSource.java,v 1.3 2003/09/19 00:38:22 glchen Exp $
 */
public class SnmpSource 
    extends Source
{
    protected String _outputPath;   // path to snmp script's output file
    protected String _scriptPath;   // path to the snmp script
    protected String _community;    // name of the AP's access community


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public SnmpSource(Options opts)
    {
        _outputPath = opts.getProperty(
            "output_path", "/tmp/snmp");
        _scriptPath = opts.getProperty(
            "script_path", "/u/glchen/work/solar/script");
        _community = opts.getProperty("community");
        if (_community == null)
        {
            throw new NullPointerException(
                "Community value is not specified.");
        }
    }


    //=======================================================================
    // Implementation of solar.service.dan.Source interface.
    //=======================================================================

    public void run()
        throws Exception
    {
        // does not do anything
    }

    public Object handleQuery(Request query)
        throws Exception
    {
        return null;
    }


    //=======================================================================
    // Main entrance.
    //=======================================================================

    /**
     *  Start the snmp source and advertise a name for it. Command line
     *  usage:
     *  <p>
     *  -host tahoe -port 5470 -name "[source=snmp] -community snmp"
     */
    public static void main(String[] args)
        throws Exception
    {/*
        Options opts = new Options();
        opts.setFromCmdLine(args);

        // get command line options
        String host = opts.getProperty("host", "localhost");
        int port = opts.getIntProperty("port", 5460);
        String name = opts.getProperty("name", "[source=snmp]");

        // create client facade object and make advertisement
        Client client = new Client(host, port, opts);
        client.advertise(name, "nullobj", new SnmpSource(opts));*/
    }
}
