/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.syslog;


import java.io.*;
import java.io.File;
import java.net.URL;
import java.util.Hashtable;
//import java.util.*;
import java.lang.String;
import java.util.StringTokenizer;
import java.lang.Integer;
import java.lang.*;

/**
 *  A mapping object holding Category of building conversion table parsed from 
 *  a data file.
 *
 *  @author     Yurong Xu 
 *  @version    $Id: APMapping.java,v 1.1 2003/08/28 18:30:07 glchen Exp $
 */
public class APMapping
{
    private Hashtable aPLocationTable;
    private Hashtable aPConfigurationTable;
    private Hashtable aPConfigurationTableByaPName;
    private Hashtable aPConfigurationTableBymACAddress;

    private int i; 
    private boolean VERBOSE = true;	


    //------------------------------------------------------------------------
    // constructors.
    //------------------------------------------------------------------------
    
    /** Constructs the table with specified local data mapping files. */
    public APMapping(String aPLocationFilename, String aPConfigurationFilename) 
    {
    	BufferedReader in;
    	aPLocationTable=new Hashtable(1000);
//    	aPLocationTable=new Hashtable();
        aPConfigurationTable= new Hashtable(1000);
        aPConfigurationTableByaPName= new Hashtable(1000);
        aPConfigurationTableBymACAddress= new Hashtable(1000);

    	try {
            
            in = new BufferedReader(new FileReader(aPLocationFilename));
    		String line;
   	      // Read a file line by line
    		while((line = in.readLine()) != null)
    		{// thanks manima
	 	        StringTokenizer st = new StringTokenizer(line,",");
	 	        
	 	        if(st.countTokens()!=5)
	 	        	continue;
	 	        String temp;
	 	        String aPName=st.nextToken();

	 	        temp=st.nextToken();
	 	        double x= (new Double(temp)).doubleValue();

	 	        temp=st.nextToken();
	 	        double y= (new Double(temp)).doubleValue();

	 	        temp=st.nextToken();
	 	        double z= (new Double(temp)).doubleValue();

	 	        temp=st.nextToken();
	 	        int floor=0;// 1,2,3,4, basement=-1, basebasement=-2
	 	        switch(temp.length())
 	        	{
	 	        	case 3:
                              char [] s= new char[2];
                              s[0]=temp.charAt(2);
                              s[1]='\0';
                              int i= temp.charAt(2) -'0';
                              String ss=new String(s);
	 	        		floor=i;
	 	        		break;
	 	        	case 4:
	 	        		floor=-2;
	 	        		break;
 	        	}
	 	        APLocation aPLocation= new APLocation(aPName,x,y,z,floor);
	 	        aPLocationTable.put(aPName,aPLocation);
    		}
    		in.close();
    	 } catch (IOException e) 
       {
            System.out.println("Cannot open the AP location file: "
                                + e.getMessage());
        }
      	
    	// read the file of AP configuration file :)
    	try {
            in = new BufferedReader(new FileReader(aPConfigurationFilename));
    		String line;
   	      // Read a file line by line
    		while((line = in.readLine()) != null)
    		{// thanks manima
	 	        // System.out.println(line); // for debug
	 	    StringTokenizer st = new StringTokenizer(line,",");
	 	    if(st.countTokens()!=10)
	 	      	continue;
	 	        
	 	    String temp;
	 	    int antennaType;
	 	    //A
	 	    String buildingName=st.nextToken();
                //B
	 	    temp=st.nextToken();
                if(temp.length()<3)
                	 continue;
	 	    if(temp.compareToIgnoreCase("dipole")==0)
	 	       antennaType=1;
	 	    else if(temp.compareToIgnoreCase("paddle")==0)
	 	       antennaType=2;
	 	    else if(temp.compareToIgnoreCase("directional")==0)
	 	       antennaType=3;
	 	    else 
                   antennaType=0;
	 	    //C
	 	    temp=st.nextToken();
	 	    String aPType=temp;
                //D
	 	    temp=st.nextToken();
	 	    String location=temp;
	 	    //E
                temp=st.nextToken();
	 	    int frequency=(new Integer(temp)).intValue();
	 	    //F
 	          temp=st.nextToken();
	 	    int power=(new Integer(temp)).intValue();
	 	    //G
	 	    String aPName=st.nextToken();
	 	    //H
 	          temp=st.nextToken();
	 	    String iPAddress=temp;
	 	    //I
	 	    temp=st.nextToken(); 
	 	    String gateway=temp;
	 	    //J
	 	    temp=st.nextToken(); 
	 	    String mACAddress=temp;

	 	    if(mACAddress.length()<=10)
	 	        continue;
	 	    APConfiguration aPConfiguration= new APConfiguration(aPName, buildingName, antennaType, aPType, location, frequency, power, iPAddress, gateway, mACAddress);
	 	    aPConfigurationTableByaPName.put(aPName,aPConfiguration);
	 	    aPConfiguration= new APConfiguration(aPName, buildingName, antennaType, aPType, location, frequency, power, iPAddress, gateway, mACAddress);
	 	    aPConfigurationTableBymACAddress.put(mACAddress,aPConfiguration);
    		}
    		in.close();
    	} catch (IOException e) 
      {
            System.out.println("Cannot open the map file: "
                                + e.getMessage());
        }
    }


    //------------------------------------------------------------------------
    // public interface methods.
    //------------------------------------------------------------------------
    
    /**
     *  Return a AP name given MAC address of AP.
     */
    public String getAPNameByMACAddress(String mACAddress)
    {
        String aPName;
        APConfiguration aPConfiguration= (APConfiguration)aPConfigurationTableBymACAddress.get(mACAddress);
        if (aPConfiguration==null)
        	return null;

    	aPName = (String) aPConfiguration.aPName;
        return aPName;
    }
    /**
     *  Return a AP configuration given MAC address of AP.
     */
    public APConfiguration getAPConfigurationByMACAddress(String mACAddress)
    {
        APConfiguration aPConfiguration= (APConfiguration)aPConfigurationTableBymACAddress.get(mACAddress);
        if (aPConfiguration==null)
        	return null;
        return aPConfiguration;
    }
    /**
     *  Return a MAC address of AP given the name of AP.
     */
    public String getMACAddressByAPName(String aPName)
    {
        String mACAddress;
        APConfiguration aPConfiguration= (APConfiguration)aPConfigurationTableByaPName.get(aPName);
        if (aPConfiguration==null)
        	return null;
    	mACAddress = (String) aPConfiguration.mACAddress;
        return mACAddress;
    }
    /**
     *  Return a AP Location clasee given the name of AP.
     */
    public APLocation getAPLocationByAPName(String aPName)
    {

    	APLocation aPLocation = (APLocation) aPLocationTable.get(aPName);
        return aPLocation;
    }
    /**
     *  Return a AP Location clasee given MAC address of AP.
     */
    public APLocation getAPLocationByMACAddress(String mACAddress)
    {
        String aPName=getAPNameByMACAddress(mACAddress);
        if (aPName==null)
        	return null;
        return getAPLocationByAPName(aPName);
    }
    /**
     *  Return a building name for AP given the MAC address of AP.
     */
    public String getAPBuildingByMACAddress(String mACAddress)
    {
        String building;
        APConfiguration aPConfiguration= (APConfiguration)aPConfigurationTableBymACAddress.get(mACAddress);
        if (aPConfiguration==null)
        	return null;
    	  building = (String) aPConfiguration.buildingName;
        return building;
    }
    /**
     *  return false, if a AP is not in the data file by MAC address of AP.
     */
    public boolean unknownAP(String macAP)
    {
        APLocation aPLocation=getAPLocationByMACAddress(macAP); 
        APConfiguration aPConfiguration=getAPConfigurationByMACAddress(macAP); 
        if ((getAPLocationByMACAddress(macAP)==null) ||(getAPConfigurationByMACAddress(macAP)==null)){
        	if( (VERBOSE == true)&&(aPLocation!=null)&&(aPConfiguration!=null)	)
               // System.out.println("here apmmaping " + aPLocation.aPName +  " configuration" + aPConfiguration.buildingName);
                System.out.println("here apmmaping " + aPLocation +  " configuration" + aPConfiguration.buildingName +"name" +aPConfiguration.aPName );


        	return true;
        	}
        return false;
    }
    
    /**
     *  Return a AP's floor # given MAC address of AP.
     */
	public int getAPFloorByMACAddress(String mACAddress)
    {
        int floor;
        APLocation aPLocation= (APLocation)aPLocationTable.get(getAPNameByMACAddress(mACAddress));
        if (aPLocation==null)
        	return -100; // -100 means no floor,
    	floor = aPLocation.floor;
        return floor;
    }
    /** Standalone test, with mapping table constructed . */
    public static void main(String[] args)
        throws Exception
    {

        APMapping aPMapping = new APMapping("aplocation.dat","apconfiguration.dat");
        System.out.println((aPMapping.getAPLocationByMACAddress("0040964088dc")).aPName+(aPMapping.getAPLocationByMACAddress("0040964088dc")).x+(aPMapping.getAPLocationByMACAddress("0040964088dc")).y+"floorno:"+(aPMapping.getAPLocationByMACAddress("0040964088dc")).floor);
        // should be apName vail5-ap, location :  820754.8575	441201.8172	0	AP4
    }
}
