/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.syslog;


import java.io.*;
import java.io.File;
import java.net.URL;
import java.util.Hashtable;
import java.lang.String;
import java.util.StringTokenizer;
import java.lang.Integer;
import java.lang.*;

/**
 *  A mapping object holding Category of building conversion table parsed from 
 *  a data file.
 *
 *  @author     Yurong Xu 
 *  @version    $Id: BuildingNameConvertor.java,v 1.1 2003/08/28 18:30:07 glchen Exp $
 */
public class BuildingNameConvertor
{
    private Hashtable catalogBuildingTable =new Hashtable(300);

    private int i; 
    private boolean VERBOSE = true;	


    //------------------------------------------------------------------------
    // constructors.
    //------------------------------------------------------------------------
    
    /** Constructs the table with specified local XML mapping file. */
    public  BuildingNameConvertor(String convertorFilename) 
    {
    	BufferedReader in;
    	
    	try {
            in = new BufferedReader(new FileReader(convertorFilename));
    		String line;
   	        // Read a file line by line
    		while((line = in.readLine()) != null)
    		    {
	 	        StringTokenizer st = new StringTokenizer(line,",");
	 	        // System.out.println(line); // for debug
	 	        if(st.countTokens()!=2)
	 	        	continue;
	 	       String buildingName= st.nextToken();
	 	       String newBuildingName =st.nextToken();
	 	       if (newBuildingName.length()<2)
	 	       	   continue;
	 	        catalogBuildingTable.put(buildingName,newBuildingName);
    		    }
    		//in.close();
    	    }
        catch (IOException e) 
      	{
            System.out.println("Cannot open the catalog file: "
                                + e.getMessage());
        }
    
     }
    public String getNewBuildingName(String buildingName)
    {
       String i =(String)catalogBuildingTable.get((Object)buildingName);
       if (i==null)
       	  return null;
       return i;
    }
    public static void main(String[] args)
        throws Exception
    {

         BuildingNameConvertor mapping = new  BuildingNameConvertor("buildingname-in-apname.csv");
    }

}
