/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.syslog;


import java.io.*;
import java.io.File;
import java.net.URL;
import java.util.Hashtable;
import java.util.*;
import java.lang.String;
import java.util.StringTokenizer;
import java.lang.Integer;
import java.lang.*;

/**
 *  A mapping object holding Category of building conversion table parsed from 
 *  a data file.
 *
 *  @author     Yurong Xu 
 *  @version    $Id: CatalogBuildingMapping.java,v 1.1 2003/08/28 18:30:09 glchen Exp $
 */
public class CatalogBuildingMapping
{
    private Hashtable catalogBuildingTable =new Hashtable(300);

    private int i; 
    private boolean VERBOSE = true;	
    private BuildingNameConvertor convertor; 


    //------------------------------------------------------------------------
    // constructors.
    //------------------------------------------------------------------------
    
   /**   Constructs the table with specified local data mapping file.
    *    this class is specifially design for Prof. Kotz 's Categories.awk's category of building 
    */
    public CatalogBuildingMapping(String catalogBuildingFilename) 
    {
     
    	BufferedReader in;

    	try {
   
            in = new BufferedReader(new FileReader(catalogBuildingFilename));
    		String line;
   	      // Read a file line by line
    		while((line = in.readLine()) != null)
    		    {
	 	        StringTokenizer st = new StringTokenizer(line,",");
	 	        Integer catalog = new Integer(100);
	 	        // System.out.println(line); // for debug
	 	        if(st.countTokens()!=2)
	 	        	continue;
	 	       String buildingName= st.nextToken();
	 	       String catalogName= new String("name");
	 	
	 	       switch ((st.nextToken()).charAt(1)){ // the 2nd char in Category
	 	        	case 'l':  // all 
	 	        		catalog= new Integer(0); 
	 	        		catalogName=new String("all");
	 	        		break; 
	 	        	case 'e': 
	 	        		catalog= new Integer(1); 
	 	        		catalogName=new String("res");
	 	        		break; 
	 	        	case 'c': 
	 	        		catalog= new Integer(2); 
	 	        		catalogName=new String("ac");
	 	        		break; 
	 	        	case 'i': 
	 	        		catalog= new Integer(3); 
	 	        		catalogName=new String("lib");
	 	        		break; 
	 	        	case 'o': 
	 	        		catalog= new Integer(4); 
	 	        		catalogName=new String("soc");
	 	        		break; 
	 	        	case 'd': 
	 	        		catalog= new Integer(5); 
	 	        		catalogName=new String("adm");
	 	        		break; 
	 	        	case 't': 
	 	        		catalog= new Integer(6); 
	 	        		catalogName=new String("ath");
	 	        		break; 
	 	        	default:
	 	        		break;
	 	        	}
	 	        catalogBuildingTable.put(buildingName,catalog);
    		    }
    		
    	    }
        catch (IOException e) 
      	{
            System.out.println("Cannot open the catalog file: "
                                + e.getMessage());
        }
    
     }

   /* public int getCatalog(String buildingName)
    {
       Integer i =(Integer)catalogBuildingTable.get((Object)buildingName);
       if (i==null)
       	return -100;
       return i.intValue();
    } */
   /** function to convert building name to category of the building. */
   public int getCategory(String aPName)
    {
       Integer i = new Integer (-100); 
       Enumeration e = catalogBuildingTable.keys();
       if (aPName==null )
       	  return -100;
       for ( ; e.hasMoreElements() ;) {
          String BuildingName=(String) e.nextElement();
          if (aPName.startsWith(BuildingName)==true)
          	{
          	 i = (Integer)catalogBuildingTable.get((Object) BuildingName);
          	 break;
          	}
       	  }
       if (i==null)
       	  return -100;
       return i.intValue();
    }

   /** just for test. */    
    public static void main(String[] args)
        throws Exception
    {
        CatalogBuildingMapping mapping = new CatalogBuildingMapping("categories.csv");
        System.out.println("this is sudi : category is " +  mapping.getCategory("sudi-3-ap"));
    }

}

