/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.syslog;


import serp.util.Options;
import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

import solar.api.Event;
import solar.api.Request;
import solar.api.Attribute;
import solar.service.dan.Operator;
import solar.impl.EventFactory;


/**
 *  An operator which added category of building and publishes it.
 *
 *  @author     Yurong Xu 
 *  @version    $Id: CategoryOperator.java,v 1.2 2003/09/19 00:38:23 glchen Exp $
 */
public class CategoryOperator 
    extends Operator
{
    private CatalogBuildingMapping mapping ;

    //=======================================================================
    // Implementation of solar.service.dan.Operator interface.
    //=======================================================================
    
    /**
     *  Init the operator's data
     *  specified in an data file.
     */
    public void init(Options opts)
        throws Exception
    {
        String catalogFilename = opts.getProperty("catalog");        
        mapping = new CatalogBuildingMapping(catalogFilename);
    }


    /** Convert aPname ->building -> category of building 
     * and public it.
     */
    public void handleEvent(Event evt) 
        throws Exception
    {
        String aPName = (String) evt.getValue("ap_name");
        
        Integer category= new Integer(-100);
        
        category = new Integer(mapping.getCategory(aPName) );
        List attrs = new Vector();
        // old one 
        attrs.add(new Attribute("timestamp", evt.getValue("timestamp")));
        attrs.add(new Attribute("msg_type", evt.getValue("msg_type")));
        Attribute attribute = new Attribute("aP_Name",aPName); 
        attrs.add(attribute);
        attrs.add(new Attribute("client_host", evt.getValue("client_host")));
        attrs.add(new Attribute("client_mac", evt.getValue("client_mac")));
        attrs.add(new Attribute("client_action", evt.getValue("client_action")));
        attrs.add(new Attribute("client_info", evt.getValue("client_info")));

        // new one 
        attribute = new Attribute("category", category.toString());//7
        attrs.add(attribute);
                                               
        
        Event new_evt = EventFactory.getPlainEvent(attrs);
        publish(new_evt);
    }


    public Object handleQuery(Request query)
        throws Exception
    {
        return null;
    }

    
    //=======================================================================
    // Main entrance for test purpose.
    //=======================================================================
    
    /** 
     *  Method used for test, not used when in Operator mode.
     */
    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setProperty("catalog","categories.csv");        
        new CategoryOperator().init(opts);
    }
}
