/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.sysnort;


import serp.util.Options;
import serp.util.Options;
import org.apache.log4j.Logger;
import org.apache.log4j.Level;
import org.apache.log4j.BasicConfigurator;
import java.io.File;
import java.io.FileInputStream;
import java.util.List;
import java.util.Vector;
import java.util.Iterator;

import solar.api.Id;
import solar.api.Event;
import solar.api.Address;
import solar.service.SolarServices;
import solar.service.ServiceManager;
import solar.service.dispatch.DispatchService;
import solar.service.pack.PackService;
import solar.service.transport.SocketAddress;
import solar.util.StringUtil;


/**
 *  A Syslog application using Solar PACK service.  This app checks each
 *  arriving event against a list of rules (as plugins) and takes specified
 *  action if the rule is satisfied.  Each rule may specify its own PACK
 *  policy.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: SysnortApp.java,v 1.12 2003/10/07 01:13:23 glchen Exp $
 */
public class SysnortApp 
{
    private static Logger _log = Logger.getLogger("solar");

    protected Options _opts;
    protected XmlConfigParser _parser;
    protected SysnortConfig _config;
    protected List _triggers;
    

    //=======================================================================
    //  Constructor.
    //=======================================================================

    public SysnortApp(Options opts)
        throws Exception
    {
        _opts = opts;
        String tgfile = _opts.getProperty("triggers", "sysnort.xml");
        _parser = new XmlConfigParser();
        _config = _parser.parse(new File(tgfile));
        _triggers = _config.loadTriggers();
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================
    
    public void run()
        throws Exception
    {
        // get basic service manager
        ServiceManager sm = SolarServices.getInstance();
        sm.init(_opts);
        // initiate services first
        DispatchService tpt = (DispatchService) sm.getService("dispatch");
        PackService pack = (PackService) sm.getService("pack");

        // get sysnort running options
        String host = _opts.getProperty("host", "localhost");
        int port = _opts.getIntProperty("port", 5460);
        Id pubid = new Id(_opts.getProperty("pubid", "SysnortSensor"));
        Address address = new SocketAddress(host, port);

        // make subscriptions for each trigger
        Iterator itr = _triggers.iterator();
        while (itr.hasNext())
        {
            Trigger trigger = (Trigger) itr.next();
            Id subid = Id.makeId("sysnort-");
            pack.attachSubscriber(subid, new TriggerHandler(trigger));
            pack.subscribe(pubid, subid, trigger.getPackPolicy());
        }

    }


    //=======================================================================
    //  Main entrance.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);

        // set the logging verbose level
        boolean debug = opts.getBooleanProperty("debug", false);
        if (debug)
        {
            _log.setLevel( (Level) Level.DEBUG );
        }
        else
        {
            _log.setLevel( (Level) Level.INFO );
        }
        BasicConfigurator.configure();

        // load configuration options
        String cfg = opts.getProperty("cfg");
        if (cfg != null)
            opts.load(new FileInputStream(cfg));
        
        new SysnortApp(opts).run();
    }
}
