/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
 
package org.doit.muffin.filter;


import org.doit.muffin.*;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.security.MessageDigest;
import java.net.InetAddress;
import solar.util.NetUtil;
import solar.util.StringUtil;


/**
 *  Factory for logging filters.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: Logger.java,v 1.2 2003/09/11 03:16:33 glchen Exp $
 */
public class Logger 
    implements FilterFactory
{
    protected FilterManager manager;
    protected Prefs prefs;
    
    protected PrintWriter _log;
    protected boolean _anonymize;
    protected MessageDigest _md;


    //=======================================================================
    // Constructors.
    //=======================================================================

    public Logger()
    {
        try
        {
            _md = MessageDigest.getInstance("MD5");
        }
        catch (Exception ex) {}
    }

    
    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public void log(Request request, long requestTime, Reply reply)
        throws Exception
    {
        // do not record anything if request or reply is missing
        if (request == null || reply == null) { return; }
        
        String SEPARATOR = " ";
        StringBuffer sb = new StringBuffer();
        // request info
        sb.append(Long.toString(requestTime) + SEPARATOR);
        sb.append(getClientIpAddress(request) + SEPARATOR);
        sb.append(request.getProtocol() + SEPARATOR);
        sb.append(request.getCommand() + SEPARATOR);
        sb.append(request.getHost() + SEPARATOR);
        sb.append(Integer.toString(request.getPort()) + SEPARATOR);
        boolean cp = request.getURL().indexOf("compass=true") != -1;
        sb.append((cp ? "compass=true" : "compass=false") + SEPARATOR);
        // reply info
        sb.append(Long.toString(System.currentTimeMillis()) + SEPARATOR);
        sb.append(reply.getProtocol() + SEPARATOR);
        sb.append(reply.getStatusCode() + SEPARATOR);
        sb.append(reply.getContentType() + SEPARATOR);
        sb.append(reply.getHeaderField("Content-length"));
        _log.println(sb.toString());
        _log.flush();        
    }

    private String getClientIpAddress(Request request)
    {
        String ip = NetUtil.getIpAddress(request.getClient().getInetAddress());
        if (ip.equals("127.0.0.1"))
        {
            try
            {
                ip = NetUtil.getIpAddress(InetAddress.getLocalHost());
            }
            catch (Exception ex) {}
        }
        if (_anonymize)
        {
            _md.update(ip.getBytes());
            ip = StringUtil.toHexString(_md.digest());
            _md.reset();
        }
        return ip;
    }


    //=======================================================================
    // Implementation of FilterFactory interface.
    //=======================================================================
    
    public void setManager(FilterManager manager)
    {
    	this.manager = manager;
    }
    
    public void setPrefs(Prefs prefs)
    {
        this.prefs = prefs;
        boolean o = prefs.getOverride();
        prefs.setOverride(false);
        prefs.putString("Logger.logfile", "compass.log");
        prefs.putString("Logger.anonymize", "true");
        prefs.setOverride(o);
        try
        {
            _log = new PrintWriter(new FileWriter(
                prefs.getString("Logger.logfile"), true));
        }
        catch (Exception ex)
        {
            throw new IllegalArgumentException(ex.toString());
        }
        _anonymize = prefs.getString("Logger.anonymize").equals("true");
    }

    public Prefs getPrefs()
    {
    	return prefs;
    }

    public void viewPrefs()
    {
    }
    
    public Filter createFilter()
    {
        Filter f = new LoggerFilter(this);
        f.setPrefs(prefs);
        return f;
    }
    
    public void shutdown()
    {
        try
        {
            _log.close();
        }
        catch (Exception ex) {}
    }

    void save()
    {
    	manager.save(this);
    }
}
