/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package rice.pastry.dist;


import rice.pastry.*;
import rice.pastry.socket.*;
import rice.pastry.wire.*;


/**
 *  A replacement of original DistPastryNodeFactory class.
 *  An abstraction of the nodeId factory for distributed nodes. In order 
 *  to obtain a nodeId factory, a client should use the getFactory method, 
 *  passing in either PROTOCOL_TCP or PROTOCOL_UDP as the protocol, and the 
 *  port number the factory should use.
 *
 *  @author     Guanling Chen
 *  @version    $Id: NewDistPastryNodeFactory.java,v 1.3 2003/10/01 00:58:40 glchen Exp $
 */
public abstract class NewDistPastryNodeFactory 
    extends DistPastryNodeFactory
{
    // choices of protocols
    public static int PROTOCOL_TCP = 0;
    public static int PROTOCOL_UDP = 1;


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    protected  NewDistPastryNodeFactory() 
    {
    }
    

    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /**
    * Static method which is designed to be used by clients needing a
    * distrubuted pastry node factory.  The protocol should be one of
    * PROTOCOL_TCP or PROTOCOL_UDP.
    *
    * @param protocol The protocol to use (PROTOCOL_TCP or PROTOCOL_UDP)
    * @param port The starting port.
    * @return A DistPastryNodeFactory using the given protocol and port.
    * @throws IllegalArgumentException If protocol is an unsupported port.
    */
    public static DistPastryNodeFactory getFactory(
        boolean serialize,
        NodeIdFactory nf, int protocol, int port) 
    {
        if (protocol == PROTOCOL_TCP)
        {
            return new TcpPastryNodeFactory(nf, port);
//            return new SocketPastryNodeFactory(serialize,
//                nf, SocketPastryNodeFactory.PROTOCOL_TCP, port);
        }
        else if (protocol == PROTOCOL_UDP)
        {
            return new UdpPastryNodeFactory(nf, port);
//            return new SocketPastryNodeFactory(serialize,
//                nf, SocketPastryNodeFactory.PROTOCOL_UDP, port);
        }
        else
        {
            throw new IllegalArgumentException(
                "Unsupported Protocol " + protocol);
        }
    }
}
