/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.dan;


import java.util.Iterator;

import solar.api.Guid;
import solar.service.dan.lang.*;
import solar.service.directory.Name;
import solar.service.fap.FapConstants;
import solar.util.NetUtil;


/**
 *  This graph manager simply loads the whole graph (except sources) onto
 *  the local Planet from which the request is received.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: LocalGraphManager.java,v 1.3 2003/09/25 15:38:51 glchen Exp $
 */
public class LocalGraphManager 
    extends GraphManager
    implements DanConstants, FapConstants
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public LocalGraphManager(GraphSpec gs)
    {
        super(gs);
    }


    //=======================================================================
    // Implementation of GraphManager interface.
    //=======================================================================

    public Guid loadOperator(String opname)
        throws DanException
    {
        FilterSpec ns = _gs.getFilter(opname);
        try
        {
            Guid opid = _dan.makeGuid();
            String clazz = ns.getClassname();
            String codebase = ns.getCodebase();
            // load the operator and register it
            Operator op = (Operator) NetUtil.loadObject(codebase, clazz);
            op.setId(opid);
            op.init(ns.getArguments());
            _nodeman.putLocal(opid, new NodeHandler(op));
            
            // load all its inputs if necessary
            Iterator itr = ns.getInputs().iterator();
            while (itr.hasNext())
            {
                PipeSpec ps = _gs.getPipe((String)itr.next());
                if (ps.isInputQuery())
                {
                    Name query = ps.getInputQuery();
                    Monitor mon = new Monitor(_dan);
                    mon.subscribe(opid, query);
                    // handle CS query
                    String root = GraphSpec.getOpVariable(query);
                    if (root != null)
                    {
                        Guid monid = _dan.makeGuid();
                        _nodeman.putLocal(monid, 
                            new NodeHandler(new ProxyOperator(mon)));
                        Guid rid = loadOperator(root);
                        //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                        // TODO: use default now, to fix the compo-lang
                        _dan.connect(FAP_PUSH, 
                            rid, DAN_OUTPORT, 
                            monid, DAN_INPORT);
                    }
                }
                else
                {
                    Guid rid = loadOperator(ps.getInputFilter());
                    _dan.connect(FAP_PUSH, 
                        rid, ps.getOutputPort(), 
                        opid, ps.getInputPort());
                }
            }
            return opid;
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            throw new DanException(ex);
        }
    }
    
}
