/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
 
package solar.service.dan;


import java.util.Set;
import java.util.HashSet;
import java.util.Hashtable;

import solar.api.Guid;
import solar.api.Address;


/**
 *  This class manages all the local graph nodes.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: NodeManager.java,v 1.3 2003/10/05 18:50:55 glchen Exp $
 */
class NodeManager 
{
    protected Set _gmans;           // a set of graph managers
    protected Hashtable _lnodes;    // a list of local nodes
    protected Hashtable _rnodes;    // a list of remote nodes


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public NodeManager()
    {
        _gmans = new HashSet();
        _lnodes = new Hashtable();
        _rnodes = new Hashtable();
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Add a graph manager containing a graph of nodes. */
    public void addGraphManager(GraphManager gm)
    {
        _gmans.add(gm);
    }

    /** Remove a graph manager from this node manager. */
    public void removeGraphManager(GraphManager gm)
    {
        _gmans.remove(gm);
    }
    

    /** Put a local node (handler) with specified guid. */
    public void putLocal(Guid guid, NodeHandler node)
    {
        _lnodes.put(guid, node);
    }

    /** Get a local node (handler) with specified guid. */
    public NodeHandler getLocal(Guid guid)
    {
        return (NodeHandler) _lnodes.get(guid);
    }


    /** Put a remove node with specified address. */
    public void putRemote(Guid guid, Address addr)
    {
        _rnodes.put(guid, addr);
    }

    /** Get the address of a remote node. */
    public Address getRemote(Guid guid)
    {
        return (Address) _rnodes.get(guid);
    }
}
