/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.pack.filter;


import cern.jet.random.Uniform;
import cern.jet.random.engine.MersenneTwister;

import serp.util.Options;
import solar.api.Event;
import solar.service.pack.PackFilter;
import solar.service.pack.PackSession;


/**
 *  This filter uniformly drop certain percent of buffered events.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: RANDOM.java,v 1.1 2003/10/13 19:34:25 glchen Exp $
 */
public class RANDOM
    extends PackFilter
{
    protected static Uniform _unigen = new Uniform(new MersenneTwister());
    
    protected int _percent;
    
    //=======================================================================
    // Implementation of solar.service.pack.PackFilter interface.
    //=======================================================================

    public void init(Options opts)
        throws Exception
    {
        String percent = opts.getProperty("percent");
        if (percent == null)
        {
            throw new IllegalArgumentException(
                "RANDOM: dropping percentage not specified.");
        }
        _percent = Integer.parseInt(percent);
    }

    public void startSession(PackSession session)
    {
        
    }
    
    public boolean filter(Event evt)
        throws Exception
    {
        return (_unigen.nextIntFromTo(1, 10000) % _percent) != 0;
    }

    public void endSession(PackSession session)
    {
        
    }
}
