/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.pack.proto;


import solar.api.Id;
import solar.api.Event;
import solar.api.Address;
import solar.impl.PlainEvent;


/**
 *  Container event output from buffer manager, for internal use instead of
 *  wired transfer.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: BufferedEvent.java,v 1.3 2003/10/05 18:50:59 glchen Exp $
 */
public class BufferedEvent
    extends PlainEvent
{
    protected Id _pubId;
    protected Id _subId;
    protected Address _dest;
    protected Event _event;
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public BufferedEvent(Id pubId, Id subId, Event evt)
    {
        _pubId = pubId;
        _subId = subId;
        _event = evt;
    }

    public BufferedEvent(Id pubId, Address dest, Event evt)
    {
        _pubId = pubId;
        _dest = dest;
        _event = evt;
    }

    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Return the ID of the publisher for this event. */
    public Id getPubId()
    {
        return _pubId;
    }

    /** If return null, then it is multicast event otherwise it is unicast. */
    public Id getSubId()
    {
        return _subId;
    }

    /** If return null, then it is multicast event otherwise it is unicast. */    
    public Address getDestination()
    {
        return _dest;
    }

    /** Return the contained event for wire transfer. */
    public Event getEvent()
    {
        return _event;
    }
}
