/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.transport;


import java.util.Random;
import rice.pastry.Id;

import solar.api.Address;


/**
 *  Network address for Pastry substrate. Note rice.pastry.messaging.Address
 *  is different and used to demulplex on the same Pastry node.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: PastryAddress.java,v 1.10 2003/10/05 18:51:03 glchen Exp $
 */
public class PastryAddress 
    implements Address      // solar.transport.Address
{
    protected Id    _nodeId;
    protected int   _hashcode;


    //=======================================================================
    //  Constructors.
    //=======================================================================

    /** Construct a Pastry address with specified Node ID. */
    public PastryAddress(Id id)
    {
        _nodeId = id;
        construction();
    }


    /** Construct a Pastry address with specified Node ID in byte array. */
    public PastryAddress(byte[] id)
    {
        _nodeId = new Id(id);
        construction();
    }


    /** Construct a Pastry address with a random Node ID. */
    public PastryAddress()
    {
    	_nodeId = Id.makeRandomId(
            new Random(System.currentTimeMillis()));
        construction();
    }


    /** Common constructor operatoions. */
    private void construction()
    {
        _hashcode = _nodeId.hashCode();
    }


    //=======================================================================
    //  Implementation of solar.service.transport.Address interface.
    //=======================================================================

    public String toString()
    {
        return _nodeId.toStringFull();
    }

    public byte[] toBytes()
    {
        byte[] cnt = _nodeId.copy();
        byte[] ret = new byte[cnt.length + 1];
        ret[0] = (byte) getEncoding();
        System.arraycopy(cnt, 0, ret, 1, cnt.length);
        return ret;
    }

    public int getEncoding()
    {
        return TransportConstants.PASTRY_ADDRESS;
    }

    public boolean equals(Object obj)
    {
        if (obj == null || !(obj instanceof PastryAddress))
            return false;
        PastryAddress other = (PastryAddress) obj;
        return _nodeId.equals(other._nodeId);
    }

    public int hashCode()
    {
        return _hashcode;
    }

    
    //=======================================================================
    //  Public interface methods.
    //=======================================================================

    /** Return the pastry node ID for this address. */
    public Id getNodeId()
    {
        return _nodeId;
    }
}
