/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.transport;


import serp.util.Options;
import org.apache.log4j.Logger;

import solar.service.ServiceException;
//import solar.service.routing.RoutingService;


/**
 *  Transport service that uses TCP, and requires a routing service.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: TcpTransportService.java,v 1.19 2003/10/12 03:56:35 glchen Exp $
 */
public class TcpTransportService 
    extends BaseTransportService
    implements TransportConstants
{
    //=======================================================================
    //  Implementation of solar.service.Service interface.
    //=======================================================================
    
    public void init()
        throws ServiceException
    {
        _log = Logger.getLogger(TcpTransportService.class);
        
        try
        {
            Options config = getServiceManager().getServiceConfig(getName());
            String hostname = config.getProperty("hostname");
            int port = config.getIntProperty("port", TRANSPORT_TCP_PORT);
            boolean serialize = config.getBooleanProperty("serialize", true);
            if (serialize)
            {
                _transport = new TcpObjectTransport(hostname, port);
            }
            else
            {
                _transport = new TcpTransport(hostname, port);
                _transport.setAduFactory(_aduFactory);
            }
            _log.debug("got transport: " + _transport);
            _transport.addTransportListener(this);
        }
        catch (ServiceException se)
        {
            throw se;
        }
        catch (Exception ex)
        {
            throw new ServiceException(ex);
        }
    }
    
    public void initDependents()
        throws ServiceException
    {
        super.initDependents();     // must call at the top
        
        // get routing service and init it with this transport service
        //_routing = (RoutingService) 
        //    getDependentService("routing", "routing");
    }

}
