/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.transport;


import solar.api.Id;
import solar.api.Adu;
import solar.api.Address;
import solar.api.AduFactory;


/**
 *  Interface for transport layer.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: Transport.java,v 1.6 2003/10/05 18:51:03 glchen Exp $
 */
public interface Transport 
{
    /** Set the factory that parse the received bytes into data unit. */
    public void setAduFactory(AduFactory factory);

    
    /** Return the local address of this transport layer. */
    public Address getLocalAddress();


    /** Return a peer address by specified key. */
    public Address getAddressByKey(String key)
        throws TransportException;


    /** Send an data unit to destination address. */    
    public void send(byte[] data, Address dest)
        throws TransportException;


    /** Send an data unit to destination address with a multiplexer. */    
    public void send(byte[] data, Address dest, Id multiplex)
        throws TransportException;


    /** Send the adu over transport to destination. */
    public void send(Adu adu, Address dest)
        throws TransportException;


    /** Send the adu over transport to destination, with a multipex ID. */
    public void send(Adu adu, Address dest, Id multiplex)
        throws TransportException;


    /** Add a transport listener. */
    public void addTransportListener(TransportListener listener);


    /** Remove a transport listener. */
    public void removeTransportListener(TransportListener listener);


    /** Stop the transport layer. */
    public void stop();
}
