/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.test;


import serp.util.Options;
import java.io.FileInputStream;

import solar.Client;
import solar.api.Request;
import solar.impl.PlainEvent;
import solar.service.dan.Source;


/**
 *  A source periodically publishes events containing current timestamp.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: ClockSource.java,v 1.12 2003/09/26 19:47:18 glchen Exp $
 */
public class ClockSource
    extends Source
{
    protected int _sleep = 0;
    
    public ClockSource(int sleep)
    {
        _sleep = sleep;
    }
    
    //=======================================================================
    // Implementation of solar.service.dan.Source interface.
    //=======================================================================

    public void run()
        throws Exception
    {
        int cnt = 0;
        while (true)
        {
            String evt = "[cnt=" + Integer.toString(cnt++) + "," + "timer=" + 
                Long.toString(System.currentTimeMillis()) + "]";
            
            publish(new PlainEvent(evt));
            if (_sleep > 0)
            {
                Thread.sleep(_sleep);
            }
        }
    }

    public Object handleQuery(Request query)
        throws Exception
    {
        return null;
    }


    //=======================================================================
    // Main entrance.
    //=======================================================================

    /**
     *  Start the clock source and advertise a name for it. Command line
     *  usage:
     *  <p>
     *  -host tahoe -port 5470 -name "[source=clock]"
     */
    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);
        // load configuration options
        String cfg = opts.getProperty("cfg");
        if (cfg != null)
        {
            opts.load(new FileInputStream(cfg));
            opts.setFromCmdLine(args);  // cmd opts overwrite opts in file
        }

        // get command line options
        String host = opts.getProperty("host", "localhost");
        int port = opts.getIntProperty("port", 5470);
        String name = opts.getProperty("name", "[source=clock]");
        int sleep = opts.getIntProperty("sleep", 2000);

        // create client facade object and make advertisement
        Client client = new Client(host, port, opts);
        client.advertise(name, "nullobj", new ClockSource(sleep));
    }
}
