/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
 
package solar.test;


import serp.util.Options;
import java.io.FileInputStream;
import java.net.InetAddress;
import java.util.Random;

import solar.api.Id;
import solar.api.Event;
import solar.impl.EventFactory;
import solar.service.ServiceManager;
import solar.service.pubsub.client.PubSubClient;
import solar.service.pubsub.Publisher;


/**
 *  A standalone event publisher using Pub/Sub service.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: ScribeSender.java,v 1.3 2003/10/05 19:46:18 glchen Exp $
 */
public class ScribeSender 
    implements Publisher
{
    protected Random _random = new Random(System.currentTimeMillis());
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public ScribeSender()
    {
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================
    
    public void run(PubSubClient client, Options opts)
        throws Exception
    {
        Id pubid = new Id(opts.getProperty("pubid", "pubsub-pub"));
        client.addPublisher(pubid, this);
        
        int cnt = 0;
        while(true)
        {
            Event evt = EventFactory.getPlainEvent(
                "[tstamp=" + Long.toString(System.currentTimeMillis()) 
                + ", counter=" + Integer.toString(cnt++)
                + ", value=" + Integer.toString(_random.nextInt(1000)) + "]");
            client.publish(pubid, evt);
            Thread.sleep(1000);
        }
    }
    

    //=======================================================================
    // Main entrance.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);

        // load configuration options
        String cfg = opts.getProperty("cfg");
        if (cfg != null)
        {
            opts.load(new FileInputStream(cfg));
            opts.setFromCmdLine(args);
        }

        // get command line options
        String host = opts.getProperty("host", "localhost");
        int port = opts.getIntProperty("port", 5470);
        // create client facade object
        PubSubClient client = new PubSubClient(host, port, opts);
        new ScribeSender().run(client, opts);
    }
}
