/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.util;


import java.util.Random;


/**
 *  Some utility methods about string operations.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: StringUtil.java,v 1.5 2003/09/07 23:18:35 glchen Exp $
 */
public class StringUtil 
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    /** Prevent object initialization. */
    private StringUtil()
    {
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    // table to convert a nibble to a hex char.
    private static char[] hexChar =
    {
    '0' , '1' , '2' , '3' ,
    '4' , '5' , '6' , '7' ,
    '8' , '9' , 'a' , 'b' ,
    'c' , 'd' , 'e' , 'f' 
    };

    /** Convert a byte array to a printable hex string. */
    public static String toHexString(byte[] b)
    {
        StringBuffer sb = new StringBuffer( b.length * 2 );
        for ( int i=0 ; i<b.length ; i++ )
        {
            // look up high nibble char
            sb.append( hexChar [ ( b[ i] & 0xf0 ) >>> 4 ] ) ;

            // look up low nibble char
            sb.append( hexChar [ b[ i] & 0x0f ] ) ;
        }
        return sb.toString() ;
    }

    /** Convert a hex string to byte array. */
    public static byte[] fromHexString(String s)
    {
        int stringLength = s.length() ;
        if ( (stringLength & 0x1) != 0 )
        {
            throw new IllegalArgumentException(
                "fromHexString requires an even number of hex characters" );
        }
        byte[] b = new byte[ stringLength / 2 ];
        for ( int i=0 ,j= 0; i< stringLength; i+= 2,j ++ )
        {
            int high= charToNibble(s.charAt ( i ));
            int low = charToNibble( s.charAt ( i+1 ) );
            b[ j ] = (byte ) ( ( high << 4 ) | low );
        }
        return b;
    }

    private static int charToNibble(char c)
    {
        if ( '0' <= c && c <= '9' )
        {
            return c - '0' ;
        }
        else if ( 'a' <= c && c <= 'f' )
        {
            return c - 'a' + 0xa ;
        }
        else if ( 'A' <= c && c <= 'F' )
        {
            return c - 'A' + 0xa ;
        }
        else
        {
            throw new IllegalArgumentException("Invalid hex character: " + c) ;
        }
    }


    private static String _prefix = "";
    private static Random ran = new Random(System.currentTimeMillis());
    private static long _nextId = 100L;

    public synchronized static String makeId()
    {
        if(_nextId >= 0x7fffffffffffffffL)
        {
            _nextId = 100 + ran.nextInt(100);
            _prefix = _prefix + ":";
        }
        else
        {
            _nextId += ran.nextInt(100);
        }
        return _prefix + _nextId;
    }

    public synchronized static String makeId(String prefix)
    {
        return prefix + ":" + makeId();
    }
}
