/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package versus;


import java.io.File;
import java.net.URL;
import java.util.Hashtable;
import java.util.Iterator;


/**
 *  A mapping object holding Versus conversion table parsed from
 *  a config file.
 *
 *  @author     Guanling Chen
 *  @version    $Id: VersusMapping.java,v 1.1 2003/07/21 21:41:43 glchen Exp $
 */
public class VersusMapping
{
    private Hashtable _floorplan;
    private Hashtable _badges;


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    /** Constructs the table with specified local XML mapping file. */
    public VersusMapping(String file) 
        throws Exception
    {
        XmlVersusParser parser = new XmlVersusParser();
        parser.parse(new File(file));
        _floorplan = parser.getFloorplan();
        _badges = parser.getBadges();
    }


    /** Constructs the table with specified remote XML mapping file. */
    public VersusMapping(URL url) 
        throws Exception
    {
        XmlVersusParser parser = new XmlVersusParser();
        parser.parse(url.openStream());
        _floorplan = parser.getFloorplan();
        _badges = parser.getBadges();
    }
    

    //=======================================================================
    // Public interface methods.
    //=======================================================================
    
    /**
     *  Return a symbolic name given a badge number, return null if
     *  not found.
     */
    public String getName(String badge)
    {
        String name;
    	name = (String) _badges.get(badge);
        return name;
    }

    
    /**
     *  Return a location name given collector and sensor numbers, return
     *  null if not found.
     */
    public String getLocation(String collector, String sensor)
    {
        String zone = null;

        Hashtable collectorTable = (Hashtable) _floorplan.get(collector);
        if (collectorTable != null)
        {
            zone = (String) collectorTable.get(sensor);
        }
        
        return zone;
    }


    public void outputZones()
    {
        Iterator itr = _floorplan.keySet().iterator();
        while (itr.hasNext())
        {
            String col = (String) itr.next();
            Hashtable ct = (Hashtable) _floorplan.get(col);
            Iterator itr1 = ct.keySet().iterator();
            while (itr1.hasNext())
            {
                String sen = (String) itr1.next();
                String zone = getLocation(col, sen);
                System.out.println(col + "_" + sen + " " + zone);
            }
        }
    }


    //=======================================================================
    // Main entrance for test purpose.
    //=======================================================================
    
    /** Standalone test, with mapping table constructed from an URL. */
    public static void main(String[] args)
        throws Exception
    {
        URL url = new URL("http://www.cs.dartmouth.edu/~solar/internal/versus/versus.xml");
        VersusMapping map = new VersusMapping(url);
        map.outputZones();
    }
}
