/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package versus.meeting;


import serp.util.Options;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;


/**
 *  Meeting agent takes a stream of versus events and uses RL policy
 *  to determine whether there is a meeting at the current moment.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: RLMeetingAgent.java,v 1.7 2003/08/01 20:06:25 glchen Exp $
 */
public class RLMeetingAgent 
{
    static final int LEARNING_STEPS = 77300;

    int _steps;
    int _maxSteps;
    BufferedReader _in;

    RLearner _learner;
    RLPolicy _policy;
    RLMeetingWorld _world;
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    /** Construct the meeting agent with a versus event source. */
    public RLMeetingAgent(int steps, BufferedReader in)
    {
        _in = in;
        _maxSteps = (steps == 0) ? LEARNING_STEPS : steps;
        _world = new RLMeetingWorld(this);
        _learner = new RLearner(_world);
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public void run()
        throws Exception
    {
        _learner.setLearningMethod(RLearner.Q_LAMBDA);
        _learner.setLambda(0.05);
        _learner.setAlpha(0.01);
        _learner.setEpsilon(0.1);
        _learner.runEpoch();    // learning util world.endState() returns true
        _policy = _learner.getPolicy();
        System.err.println("Training finished, starting prediction...");

        int start=0;        
        // start prediction
        while (true)
        {
            String line = _in.readLine();
            String[] fields = line.split(" ");
            int[] state = new int[4];
            int time = Integer.parseInt(fields[2]);
            state[0] = Integer.parseInt(fields[3]);   // num of moving badges
            state[1] = Integer.parseInt(fields[4]);   // sum of sigmoids
            int action = _policy.getBestAction(state);

            if (action == 1)
            {
                if (start == 0) start = time;
            }
            else
            {
                if (start != 0)
                {
                    System.out.println(start + " " + time);
                    start = 0;
                }
            }
        }
    }


    public int[] getNextEvent()     // called by MeetingWorld
    {
        int[] ret = new int[6];
        try
        {
            String line = _in.readLine();
            String[] fields = line.split(" ");
        
            ret[0] = Integer.parseInt(fields[2]);   // timestamp in seconds
            ret[1] = Integer.parseInt(fields[3]);   // num of moving badges
            ret[2] = Integer.parseInt(fields[4]);   // sum of sigmoids
            ret[3] = Integer.parseInt(fields[5]);   // meeting or not
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
        
        _steps ++;
        if (_steps % 100 == 0)
            System.err.println("learning step: " + _steps);
        
        if(_steps > _maxSteps)
        {
            _world.setEndState(true);       // finish learning
        }
        return ret;
    }
    
    
    //=======================================================================
    // Main entrance.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);

        int steps = opts.getIntProperty("steps", 0);
        String input = opts.getProperty("input");
        InputStream in = (input == null) ? 
            System.in : new FileInputStream(input);
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        try
        {
            new RLMeetingAgent(steps, reader).run();
        }
        catch (Exception ex)
        {
            
        }
    }
}
